/*************************************************************************************************************
/ IEC60870 RTU
/ Copyright by Panasonic Electric Works Europe AG 2003-2013
/ RT 16.04.2012
**************************************************************************************************************/

//Global variables / parameters
var m_iIEC870_RBM;									//Address (DT area) calculated from IEC buffer header
var m_sIEC870_RBM1="DT12100";			//Full PLC address of the first element of the IEC buffer header
var m_sIEC870_RBM2="DT21050";			//Full PLC address of the first element of the IEC buffer header (redundancy)
var m_sIEC870_RBM2EM1="DT31426";	//Full PLC address of the first element of the IEC buffer header (EM1 unit)
var m_sSetTime="DDT4093"; //Address of variable for synchronizing time
var m_Diagnosis="WR210";			//Full PLC address of the first element of the diagnosis buffer
var m_DiagnosisDT										//First diagnosis address (WR area) calculated from g_dutDiagnosis
var m_iSizeOfRecord = 21;							//Number of words to read
var m_iLoopInterval;									//Checks the transfer status and start the next download of data
var m_bDownloadOK;									//Handshake for data download
var m_interval = 450;								//Main interval to update the data
var m_iError = 0;
var NOT_UPDATED = 9999;								//Default value to detect PLC Online;
var m_dutSetting_iUseDST = NOT_UPDATED;		//Used for Timesync
var m_dutSetting_iTimeZone = NOT_UPDATED;		//Used for Timesync
//System global 
var PLCreg = new Array(30);						//Response data array (16 bit words or integers)
var XML_Data;											//Buffer for MEWTOCOL request command
var PLC_StationAddress = "EE";					//PLC MEWTOCOL station address ("EE", "01"..."64")
var zeroes="00000";									//For generating leading zeroes in global.js

var sGray = 	"0";									//Bullet color
var sGreen = 	"1";									//Bullet color
var sOrange = 	"2";									//Bullet color
var sRed = 		"3";									//Bullet color
var GET_PLC_SYSDATA = 1;							//Read WR900

var m_sSizeOK = "Configuration ok";												//Buffer information (ok text)
var m_sSizeWarning = "Warning: Max. number of ";							//Buffer information (starting text)
var m_sSizeWarning_SP = "single point values (SP/DP) achieved";			//Buffer information
var m_sSizeWarning_ME = "measurement values (ME/M_BO) achieved";		//Buffer information
var m_sSizeWarning_IT = "integrated totels/counters (IT) achieved";	//Buffer information
var m_sSizeWarning_SC = "commands (SC/DC) achieved";						//Buffer information
var m_sSizeWarning_SE = "setpoints (SE/C_BO) achieved";					//Buffer information
var m_sDefaultBulletText = "Green if detected otherwise gray";					//Buffer information

//JavaScript started on page load
window.onload = function()
{
	$("ajaxButton_ResetBuffer1").onclick = function(){SetData(1);};		//Reset buffer
	$("ajaxButton_ResetBuffer2").onclick = function(){SetData(2);};		//Reset buffer
	$("ajaxButton_SyncWithPC_Time").onclick = function(){SyncWithPC_Time();};	//Synchronize PLC clock
	
	m_bDownloadOK = 1;								//Start download
	m_iLoopInterval = 0;								//Task manager
	
	m_DiagnosisDT = parseInt(m_Diagnosis.substring(2,m_Diagnosis.length));	//String "WR210" --> 210
	
	intT = setInterval(function(){CheckIntervall();},m_interval);	//Set update interval	
}

//Main function for data update
function CheckIntervall()
{
	$("ajaxDebug1").innerHTML = m_iLoopInterval;
	//$("ajaxDebug2").innerHTML = m_iLoopInterval;
	//$("ajaxDebug3").innerHTML = m_iError;
	//$("ajaxDebug4").innerHTML = m_iLoopInterval;

	//debugger;

	if (m_bDownloadOK == 2)	//Reset or error
	{
		m_bDownloadOK = 1;
	}
	if (m_bDownloadOK == 1)
	{
			m_bDownloadOK = 0;		// Reset flag to recognize errors
			if (m_iLoopInterval & 0x01 == GET_PLC_SYSDATA)
				XML_Data = GenerateRC(PLC_StationAddress, 900, 3, 'R'); //RCC R9000-R902F
			else
				XML_Data = GenerateRC(PLC_StationAddress, m_DiagnosisDT, m_iSizeOfRecord, 'R'); //RCC
			UpdateData(XML_Data);	// Request new data
	}
	if (m_bDownloadOK > 0)
	{
		m_bDownloadOK = 0;
		UpdateData(XML_Data);	// Request new data
	}
}


//This function checks the status of the xhr object and calls the corresponding routines to
//display the variables
function DisplayPLCdata()
{
	var sVal;
	var plc_RTC = new Date();

	if(xhr.readyState == XHR_READYSTATE_COMPLETED)
	{
		if(xhr.status == XHR_STATUS_OK)
		{
			try {
				if (XML_Data.indexOf(".xml") > 0)
					plc = eval("(" + xhr.responseText + ")");//Get PLC data
				else
					decodeRD_BCD(xhr.responseText); //RD response string, true=integer false=word
			}
			catch(e)
			{
				doError(e + "\nPLC or JSON read error.");
				return;
			}
			
			if (m_iLoopInterval & 0x01 == GET_PLC_SYSDATA)
			{
				PLC_Diagnosis = parseInt(PLCreg[0], 16);
				bPLC_IO = (PLC_Diagnosis&0x10?sRed:sGreen);				//R9004
				bPLC_BatteryError = (PLC_Diagnosis&0x20?sRed:sGreen);	//R9005

				PLC_Diagnosis = parseInt(PLCreg[2], 16);
				bPLC_Running = (PLC_Diagnosis&0x01?sGreen:sRed);		//R9020
				
				DisplayIcon("ajaxPLC_IO", bPLC_IO, "punkt", 13, 13, "error, if red");
				DisplayIcon("ajaxPLC_BatteryError", bPLC_BatteryError, "punkt", 13, 13, "error, if red");
				DisplayIcon("ajaxPLC_Running", bPLC_Running, "punkt", 13, 13, "run/prog mode");
			}
			else
			{
				//The following lines set the dynamic values into the HTML document
				$("ajaxIEC870_RBM1_BufferSize").innerHTML = parseInt(PLCreg[0], 16);
				$("ajaxIEC870_RBM1_ResetCountdown").innerHTML = parseInt(PLCreg[1], 16);
				$("ajaxIEC870_RBM1_Counter").innerHTML = parseInt(PLCreg[2], 16);
				$("ajaxIEC870_RBM1_CounterMax").innerHTML = parseInt(PLCreg[3], 16);
				
				$("ajaxIEC870_RBM2_BufferSize").innerHTML = parseInt(PLCreg[4], 16);
				$("ajaxIEC870_RBM2_ResetCountdown").innerHTML = parseInt(PLCreg[5], 16);
				$("ajaxIEC870_RBM2_Counter").innerHTML = parseInt(PLCreg[6], 16) + 65536*parseInt(PLCreg[7], 16);
				$("ajaxIEC870_RBM2_CounterMax").innerHTML = parseInt(PLCreg[8], 16) + 65536*parseInt(PLCreg[9], 16);
				
				$("ajaxIEC60870R_Version").innerHTML = parseInt(PLCreg[16], 16) + 65536*parseInt(PLCreg[17], 16);
				$("ajaxIEC60870_RTU_Version").innerHTML = parseInt(PLCreg[14], 16) + 65536*parseInt(PLCreg[15], 16);
				
				var IEC_Debug = parseInt(PLCreg[18], 16);
				
				IEC_Debug_Reset = (IEC_Debug&0x01?sGreen:sGray); 			//Gray/Green
				IEC_Debug_InterrogationA = (IEC_Debug&0x02?sGreen:sGray);//Gray/Green
				IEC_Debug_InterrogationC = (IEC_Debug&0x04?sGreen:sGray);//Gray/Green
				IEC_Debug_TestCommand = (IEC_Debug&0x08?sGreen:sGray);	//Gray/Green
				IEC_Debug_Clocksync = (IEC_Debug&0x10?sGreen:sGray);		//Gray/Green
				IEC_Debug_Command = (IEC_Debug&0x20?sGreen:sGray);			//Gray/Green
				IEC_Debug_Setpoint = (IEC_Debug&0x40?sGreen:sGray);		//Gray/Green
				IEC_Debug_ProcessReset = (IEC_Debug&0x80?sRed:sGreen);	//Green/Red
				IEC_Debug_ReadData = (IEC_Debug&0x0100?sGreen:sGray);		//Gray/Green
				
				var IEC_Bits = parseInt(PLCreg[19], 16);
				
				IEC_Bits_BufferError = (IEC_Bits&0x01?sRed:sGreen);		//Green/Red
				IEC_Bits_DaylightSavingTime = (IEC_Bits&0x02?1:0);			//1/0
				IEC_Bits_TimeValid = (IEC_Bits&0x04?sGreen:sRed);			//Red/Green
				IEC_Bits_GA_ME_not_ready = (IEC_Bits&0x08?sRed:sGray);	//Red/Gray
				IEC_Bits_GA_IT_not_ready = (IEC_Bits&0x10?sRed:sGray);	//Red/Gray
				IEC_Bits_Init_ok = (IEC_Bits&0x20?sGreen:sOrange);			//Green/Orange
				IEC_Bits_StartDT1 = (IEC_Bits&0x40?sGreen:sOrange);		//Green/Orange
				IEC_Bits_StartDT2 = (IEC_Bits&0x80?sGreen:sOrange);		//Green/Orange
				
				m_dutSetting_iUseDST = parseInt(PLCreg[10], 16);
				m_dutSetting_iTimeZone = parseInt(PLCreg[11], 16);
				var g_dutIEC60870_SaveTime_diUnixTime = parseInt(PLCreg[12], 16) + 65536*parseInt(PLCreg[13], 16);
				var g_diIEC60870_RTU_Version = parseInt(PLCreg[14], 16) + 65536*parseInt(PLCreg[15], 16);
				var g_diIEC60870R_Version = parseInt(PLCreg[16], 16) + 65536*parseInt(PLCreg[17], 16);
				if (m_dutSetting_iTimeZone > 1300) //PC time is local time to transfer
					plc_RTC.setTime(g_dutIEC60870_SaveTime_diUnixTime*1000);
				else
					plc_RTC.setTime(g_dutIEC60870_SaveTime_diUnixTime*1000+m_dutSetting_iTimeZone*36000+IEC_Bits_DaylightSavingTime*3600000);
				sVal = plc_RTC.toGMTString();
				$("ajaxg_sTimeDatePLC").innerHTML = sVal.substring(0, sVal.length-4); //Do not display GMT or UTC
				
				DisplayIcon("ajaxIEC_Debug_Reset", IEC_Debug_Reset, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_InterrogationA", IEC_Debug_InterrogationA, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_InterrogationC", IEC_Debug_InterrogationC, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_TestCommand", IEC_Debug_TestCommand, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_Clocksync", IEC_Debug_Clocksync, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_Command", IEC_Debug_Command, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_Setpoint", IEC_Debug_Setpoint, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_ProcessReset", IEC_Debug_ProcessReset, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Debug_ReadData", IEC_Debug_ReadData, "punkt", 13, 13);
				
				DisplayIcon("ajaxIEC_Bits_BufferError1", IEC_Bits_BufferError, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Bits_BufferError2", IEC_Bits_BufferError, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Bits_DaylightSavingTime", IEC_Bits_DaylightSavingTime, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Bits_TimeValid", IEC_Bits_TimeValid, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Bits_GA_ME_not_ready", IEC_Bits_GA_ME_not_ready, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Bits_GA_IT_not_ready", IEC_Bits_GA_IT_not_ready, "punkt", 13, 13);
				DisplayIcon("ajaxIEC_Bits_Init_ok", IEC_Bits_Init_ok, "punkt", 13, 13, "orange: pending, green: communication ready");
				DisplayIcon("ajaxIEC_Bits_StartDT1", IEC_Bits_StartDT1, "punkt", 13, 13, "orange: pending, green: online");
				DisplayIcon("ajaxIEC_Bits_StartDT2", IEC_Bits_StartDT2, "punkt", 13, 13, "orange: pending, green: online");
				
				Size_Warning = parseInt(PLCreg[20], 16);
				var sSizeWarning = "";
				if (Size_Warning & 0x01 != 0)
					sSizeWarning = m_sSizeWarning + m_sSizeWarning_SP;
				if (Size_Warning & 0x02 != 0)
					sSizeWarning = m_sSizeWarning + m_sSizeWarning_ME;
				if (Size_Warning & 0x04 != 0)
					sSizeWarning = m_sSizeWarning + m_sSizeWarning_IT;
				if (Size_Warning & 0x08 != 0)
					sSizeWarning = m_sSizeWarning + m_sSizeWarning_SC;
				if (Size_Warning & 0x10 != 0)
					sSizeWarning = m_sSizeWarning + m_sSizeWarning_SE;
				DisplayText("ajaxSizeWarning", sSizeWarning.length, "#008000", "#FF0000", m_sSizeOK, sSizeWarning);
			}
			m_iLoopInterval = m_iLoopInterval + 1;		//Successful polling 
			m_bDownloadOK = 1;								//Start download
		}//end if XHR_STATUS_OK
		else
		{
			m_iError = m_iError + 1; //Read error
			if (m_iError > 5)
			{
				m_iError = 0; //Error reset
				alert("Message returned, but with error status.");
			}
			bDownloadOK = 2;
		}//end else
	}//end if XHR_READYSTATE_COMPLETED
}


//This function sends the value 0 to the buffer organization structures as reset command
function SetData(iMode)
{
	if (iMode == 1)
	{
		var sPostmessage = "";
		var iIEC870_RBM;
		
		//Get the address of the buffer
		iIEC870_RBM = parseInt(m_sIEC870_RBM1.substring(2,m_sIEC870_RBM1.length));
		
		if (isNaN(iIEC870_RBM) != true)
		{
			SendDT(iIEC870_RBM, 0);
		}
	}
	
	if (iMode == 2)
	{
		var sPostmessage = "";
		var iIEC870_RBM;
		
		//Get the address of the buffer RBM2
		iIEC870_RBM = parseInt(m_sIEC870_RBM2.substring(2,m_sIEC870_RBM2.length));
		
		if (isNaN(iIEC870_RBM) != true)
		{
			sPostmessage = CreatePost_DT(iIEC870_RBM,0);
		
			//Get the address of the buffer EM1
			iIEC870_RBM = parseInt(m_sIEC870_RBM2EM1.substring(2,m_sIEC870_RBM2EM1.length));

			if (isNaN(iIEC870_RBM) != true)
			{
				sPostmessage += "&" + CreatePost_DT(iIEC870_RBM,0);
			}
			PostNow(sPostmessage);
		}
	}
}


//This function reads the value entered in the formular field and assigns it to the corresponding DTs
function SyncWithPC_Time()
{
	if (m_dutSetting_iTimeZone != NOT_UPDATED && m_dutSetting_iUseDST != NOT_UPDATED)
	{
		var diCurrentTime = new Date(); //Unix time value in seconds needed
		var iOffset = diCurrentTime.getTimezoneOffset();
		var iSetTime = parseInt(m_sSetTime.substring(3,m_sSetTime.length));
		if (m_dutSetting_iTimeZone > 1300) //PC time is local time to transfer
		{
			diCurrentTime = diCurrentTime - iOffset*60000;
		}
		Send_UI_DDT(iSetTime, diCurrentTime/1000);
		m_bDownloadOK = 2;
	}
}


//Submit a message with values to the PLC
function PostNow(sPostMessage)
{
	xhr_SendDT = createXMLHttpRequest();

	xhr_SendDT.onreadystatechange = function()		// Details see "C.) Programming Examples..."
	{
		if (xhr_SendDT.readyState == XHR_READYSTATE_COMPLETED)
	{
			if (xhr_SendDT.status != XHR_STATUS_OK)	//Check http answer
				doError(m_TextErrorPost);
		}
	};

	xhr_SendDT.open("POST","/plcpost",true);			//Open a http POST connect
	xhr_SendDT.send(sPostMessage);						// and submit CGI parameter
}


//Prepare submit message for 16 bit integer value to PLC DT address
function CreatePost_DT(address, value)
{
	var add = "0" + address;								// Generate PLC submit string,
	var adl = add.length;									// for example: DT123 <-- 456
	var par = "{DT_" + add.substring(0,adl-1) + "_" + add.substring(adl-1,adl);
	par = par + "_770_%i}=" + value;	// result: "{DT_12_3_770_%i}=456"
	return par;
}


/* End of file */